@@ -1,5 +1,5 @@
 /*
- * @(#)$Id: acconfig.h,v 1.44 2009/04/10 18:46:11 jnelson Exp $
+ * @(#)$Id: acconfig.h,v 1.46 2014/12/26 15:26:41 jnelson Exp $
  */
 
 /*
@@ -366,19 +366,19 @@
 #undef RUBYCFLAGS
 
 /* Define this if you have ruby and stuff */
-#undef RUBYLDFLAGS
+#undef RUBYLIBS
 
 /* Define this if you have tcl and stuff */
 #undef TCLCFLAGS
 
 /* Define this if you have tcl and stuff */
-#undef TCLLDFLAGS
+#undef TCLLIBS
 
 /* Define this if you have perl and stuff */
 #undef PERLCFLAGS
 
 /* Define this if you have perl and stuff */
-#undef PERLLDFLAGS
+#undef PERLLIBS
 
 /* Define this if you have finite() */
 #undef HAVE_FINITE
@@ -428,3 +428,15 @@
 /* Define this if you need strtoll() */
 #undef NEED_STRTOLL
 
+/* Define this if you have <termios.h> */
+#undef HAVE_TERMIOS_H
+
+/* Define this if you have <sys/termios.h> */
+#undef HAVE_SYS_TERMIOS_H
+
+/* Define this if you have <xlocale.h> */
+#undef HAVE_XLOCALE_H
+
+/* Define this if you have newlocale() */
+#undef HAVE_NEWLOCALE
+
