@@ -74,6 +74,7 @@ enum HOOK_TYPES {
 	PUBLIC_NOTICE_LIST,
 	PUBLIC_OTHER_LIST,
 	RAW_IRC_LIST,
+	RAW_IRC_BYTES_LIST,
 	REDIRECT_LIST,
 	SEND_ACTION_LIST,
 	SEND_CTCP_LIST,
@@ -131,8 +132,4 @@ enum HOOK_TYPES {
 
 	extern int deny_all_hooks;
 
-#define ADD_STR_TO_LIST(str, sep, item, len) if (len == 0) \
-	{str = malloc_strdup(item); len = strlen(str);} \
-	else malloc_strcat_wordlist_c(&str, sep, item, &len);
-	
 #endif /* __hook_h_ */
