@@ -1,11 +1,10 @@
-/* $EPIC: crypto.c,v 1.17 2010/04/02 23:14:45 jnelson Exp $ */
 /*
  * crypto.c: SED/CAST5/BLOWFISH/AES encryption and decryption routines.
  *
  * Copyright (c) 1990 Michael Sandroff.
  * Copyright (c) 1991, 1992 Troy Rollo.
  * Copyright (c) 1992-1996 Matthew Green.
- * Copyright  2006 EPIC Software Labs
+ * Copyright 2006, 2015 EPIC Software Labs
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -163,22 +162,23 @@
 #endif
 
 #ifdef HAVE_SSL
-static char *	decipher_evp (const unsigned char *key, int keylen, const unsigned char *ciphertext, int cipherlen, const EVP_CIPHER *type, int *outlen, int ivsize);
+static char *	decipher_evp (const unsigned char *passwd, int passwdlen, const unsigned char *ciphertext, int cipherlen, const EVP_CIPHER *type, int *outlen, int ivsize);
 #endif
-static char *	decrypt_by_prog (const unsigned char *str, size_t *len, Crypt *key);
+static char *	decrypt_by_prog (const unsigned char *str, size_t *len, Crypt *crypt);
 
 #ifdef HAVE_SSL
-static char *	cipher_evp (const unsigned char *key, int keylen, const unsigned char *plaintext, int plaintextlen, const EVP_CIPHER *type, int *retsize, int ivsize);
+static char *	cipher_evp (const unsigned char *passwd, int passwdlen, const unsigned char *plaintext, int plaintextlen, const EVP_CIPHER *type, int *retsize, int ivsize);
 #endif
-static char *	encrypt_by_prog (const unsigned char *str, size_t *len, Crypt *key);
+static char *	encrypt_by_prog (const unsigned char *str, size_t *len, Crypt *crypt);
 
-unsigned char *	decipher_message (const unsigned char *ciphertext, size_t len, Crypt *key, int *retlen)
+/* This function is used by CTCP handling, but not by xform! */
+unsigned char *	decipher_message (const unsigned char *ciphertext, size_t len, Crypt *crypti, int *retlen)
 {
     do
     {
-	if (key->type == CAST5CRYPT || key->type == BLOWFISHCRYPT ||
-	    key->type == AES256CRYPT || key->type == AESSHA256CRYPT ||
-	    key->type == FISHCRYPT)
+	if (crypti->sed_type == CAST5CRYPT || crypti->sed_type == BLOWFISHCRYPT ||
+	    crypti->sed_type == AES256CRYPT || crypti->sed_type == AESSHA256CRYPT ||
+	    crypti->sed_type == FISHCRYPT)
 	{
 	    unsigned char *	outbuf = NULL;
 #ifdef HAVE_SSL
@@ -186,19 +186,19 @@ unsigned char *	decipher_message (const unsigned char *ciphertext, size_t len, C
 	    int	bytes_to_trim;
 	    int ivsize, blocksize;
 
-	    if (key->type == CAST5CRYPT)
+	    if (crypti->sed_type == CAST5CRYPT)
 	    {
 		ivsize = 8, blocksize = 8;
 	    }
-	    else if (key->type == BLOWFISHCRYPT)
+	    else if (crypti->sed_type == BLOWFISHCRYPT)
 	    {
 		ivsize = 8, blocksize = 8;
 	    }
-	    else if (key->type == FISHCRYPT)
+	    else if (crypti->sed_type == FISHCRYPT)
 	    {
 		ivsize = 0, blocksize = 8;
 	    }
-	    else if (key->type == AES256CRYPT || key->type == AESSHA256CRYPT)
+	    else if (crypti->sed_type == AES256CRYPT || crypti->sed_type == AESSHA256CRYPT)
 	    {
 		ivsize = 16, blocksize = 16;
 	    }
@@ -207,29 +207,34 @@ unsigned char *	decipher_message (const unsigned char *ciphertext, size_t len, C
 
 	    if (blocksize > 0 && len % blocksize != 0)
 	    {
-		yell("Encrypted message [%s] isn't multiple of %d! (is %d)", 
-				ciphertext, blocksize, len);
+		/* 
+		 * XXX Sigh, you can't portably printf() a size_t, since 
+		 * maybe it's an int, maybe it's a long, and any printf
+		 * format that DTRT is too new to trust.  bleh.
+		 */
+		yell("Encrypted message [%s] isn't multiple of %d! (is %ld)", 
+				ciphertext, blocksize, (long)len);
 		break;
 	    }
 	    if ((int)len < blocksize + ivsize)
 	    {
 		yell("Encrypted message [%s] doesn't contain message! "
-				"(len is %d)", ciphertext, len);
+				"(len is %ld)", ciphertext, (long)len);
 		break;
 	    }
 
-	    if (key->type == CAST5CRYPT)
+	    if (crypti->sed_type == CAST5CRYPT)
 		type = EVP_cast5_cbc();
-	    else if (key->type == BLOWFISHCRYPT)
+	    else if (crypti->sed_type == BLOWFISHCRYPT)
 		type = EVP_bf_cbc();
-	    else if (key->type == FISHCRYPT)
+	    else if (crypti->sed_type == FISHCRYPT)
 		type = EVP_bf_ecb();
-	    else if (key->type == AES256CRYPT || key->type == AESSHA256CRYPT)
+	    else if (crypti->sed_type == AES256CRYPT || crypti->sed_type == AESSHA256CRYPT)
 		type = EVP_aes_256_cbc();
 	    else
 		break;		/* Not supported */
 
-	    if (!(outbuf = decipher_evp(key->key, key->keylen,
+	    if (!(outbuf = decipher_evp(crypti->passwd, crypti->passwdlen,
 					ciphertext, len, 
 					type, retlen, ivsize)))
 	    {
@@ -244,26 +249,26 @@ unsigned char *	decipher_message (const unsigned char *ciphertext, size_t len, C
 #endif
 	    return outbuf;
 	}
-	else if (key->type == SEDCRYPT || key->type == SEDSHACRYPT)
+	else if (crypti->sed_type == SEDCRYPT || crypti->sed_type == SEDSHACRYPT)
 	{
 		unsigned char *	text;
 
 		text = new_malloc(len + 1);
 		memmove(text, ciphertext, len);
-		decrypt_sed(text, len, key->key, key->keylen);
+		decrypt_sed(text, len, crypti->passwd, crypti->passwdlen);
 		*retlen = len;
 		return text;
 	}
-	else if (key->type == PROGCRYPT)
+	else if (crypti->sed_type == PROGCRYPT)
 	{
 		unsigned char *retval;
 
-		retval = decrypt_by_prog(ciphertext, &len, key);
+		retval = decrypt_by_prog(ciphertext, &len, crypti);
 		*retlen = len;
 		return retval;
 	}
 	else
-		panic(1, "decipher_message: key->type %d is not valid", key->type);
+		panic(1, "decipher_message: crypti->sed_type %d is not valid", crypti->sed_type);
     }
     while (0);
 
@@ -271,11 +276,12 @@ unsigned char *	decipher_message (const unsigned char *ciphertext, size_t len, C
 }
 
 #ifdef HAVE_SSL
-static char *	decipher_evp (const unsigned char *key, int keylen, const unsigned char *ciphertext, int cipherlen, const EVP_CIPHER *type, int *outlen, int ivsize)
+static char *	decipher_evp (const unsigned char *passwd, int passwdlen, const unsigned char *ciphertext, int cipherlen, const EVP_CIPHER *type, int *outlen, int ivsize)
 {
         unsigned char *outbuf;
 	unsigned char	*iv = NULL;
 	unsigned long errcode;
+	int	outlen2;
         EVP_CIPHER_CTX a;
         EVP_CIPHER_CTX_init(&a);
 	EVP_CIPHER_CTX_set_padding(&a, 0);
@@ -287,9 +293,16 @@ static char *	decipher_evp (const unsigned char *key, int keylen, const unsigned
 		memcpy(iv, ciphertext, ivsize);
 
         EVP_DecryptInit_ex(&a, type, NULL, NULL, iv);
-	EVP_CIPHER_CTX_set_key_length(&a, keylen);
-        EVP_DecryptInit_ex(&a, NULL, NULL, key, NULL);
-        EVP_DecryptUpdate(&a, outbuf, outlen, ciphertext, cipherlen);
+	EVP_CIPHER_CTX_set_key_length(&a, passwdlen);
+	EVP_CIPHER_CTX_set_padding(&a, 0);
+        EVP_DecryptInit_ex(&a, NULL, NULL, passwd, NULL);
+
+        if (EVP_DecryptUpdate(&a, outbuf, outlen, ciphertext, cipherlen) != 1)
+		yell("EVP_DecryptUpdate died.");
+	if (EVP_DecryptFinal_ex(&a, outbuf + (*outlen), &outlen2) != 1)
+		yell("EVP_DecryptFinal_Ex died.");
+	*outlen += outlen2;
+
         EVP_CIPHER_CTX_cleanup(&a);
 
 	ERR_load_crypto_strings();
@@ -306,47 +319,47 @@ static char *	decipher_evp (const unsigned char *key, int keylen, const unsigned
 }
 #endif
 
-void     decrypt_sed (unsigned char *str, int len, const unsigned char *key, int key_len)
+void     decrypt_sed (unsigned char *str, int len, const unsigned char *passwd, int passwdlen)
 {
-        int	key_pos,
+        int	passwd_pos,
                 i;
         char    mix,
                 tmp;
 
-        if (!key)
+        if (!passwd)
                 return;                 /* no decryption */
 
-        key_len = strlen(key);
-        key_pos = 0;
+        passwdlen = strlen(passwd);
+        passwd_pos = 0;
         mix = 0;
 
         for (i = 0; i < len; i++)
         {
-                tmp = mix ^ str[i] ^ key[key_pos]; 
+                tmp = mix ^ str[i] ^ passwd[passwd_pos]; 
                 str[i] = tmp;
                 mix ^= tmp;
-                key_pos = (key_pos + 1) % key_len;
+                passwd_pos = (passwd_pos + 1) % passwdlen;
         }
         str[i] = (char) 0;
 }
 
-static char *	decrypt_by_prog (const unsigned char *str, size_t *len, Crypt *key)
+static char *	decrypt_by_prog (const unsigned char *str, size_t *len, Crypt *crypti)
 {
         char    *ret = NULL, *input;
         char *  args[3];
         int     iplen;
 
-        args[0] = malloc_strdup(key->prog);
+        args[0] = malloc_strdup(crypti->prog);
         args[1] = malloc_strdup("decrypt");
         args[2] = NULL;
-        input = malloc_strdup2(key->key, "\n");
+        input = malloc_strdup2(crypti->passwd, "\n");
 
         iplen = strlen(input);
         new_realloc((void**)&input, *len + iplen);
         memmove(input + iplen, str, *len);
 
         *len += iplen;
-        ret = exec_pipe(key->prog, input, len, args);
+        ret = exec_pipe(crypti->prog, input, len, args);
 
         new_free(&args[0]);
         new_free(&args[1]);
@@ -358,38 +371,38 @@ static char *	decrypt_by_prog (const unsigned char *str, size_t *len, Crypt *key
 }
 
 /*************************************************************************/
-unsigned char *	cipher_message (const unsigned char *orig_message, size_t len, Crypt *key, int *retlen)
+unsigned char *	cipher_message (const unsigned char *orig_message, size_t len, Crypt *crypti, int *retlen)
 {
 	if (retlen)
 		*retlen = 0;
-	if (!orig_message || !key || !retlen)
+	if (!orig_message || !crypti || !retlen)
 		return NULL;
 
-	if (key->type == CAST5CRYPT || key->type == BLOWFISHCRYPT ||
-	    key->type == FISHCRYPT ||
-	    key->type == AES256CRYPT || key->type == AESSHA256CRYPT)
+	if (crypti->sed_type == CAST5CRYPT || crypti->sed_type == BLOWFISHCRYPT ||
+	    crypti->sed_type == FISHCRYPT ||
+	    crypti->sed_type == AES256CRYPT || crypti->sed_type == AESSHA256CRYPT)
 	{
 	    unsigned char *ciphertext = NULL;
 #ifdef HAVE_SSL
 	    size_t	ivlen;
 	    const EVP_CIPHER *type;
 
-	    if (key->type == CAST5CRYPT)
+	    if (crypti->sed_type == CAST5CRYPT)
 	    {
 		type = EVP_cast5_cbc();
 		ivlen = 8;
 	    }
-	    else if (key->type == BLOWFISHCRYPT)
+	    else if (crypti->sed_type == BLOWFISHCRYPT)
 	    {
 		type = EVP_bf_cbc();
 		ivlen = 8;
 	    }
-	    else if (key->type == FISHCRYPT)
+	    else if (crypti->sed_type == FISHCRYPT)
 	    {
 		type = EVP_bf_ecb();
 		ivlen = 0;		/* XXX Sigh */
 	    }
-	    else if (key->type == AES256CRYPT || key->type == AESSHA256CRYPT)
+	    else if (crypti->sed_type == AES256CRYPT || crypti->sed_type == AESSHA256CRYPT)
 	    {
 		type = EVP_aes_256_cbc();
 		ivlen = 16;
@@ -397,7 +410,7 @@ unsigned char *	cipher_message (const unsigned char *orig_message, size_t len, C
 	    else
 		return NULL;	/* Not supported */
 
-	    if (!(ciphertext = cipher_evp(key->key, key->keylen,
+	    if (!(ciphertext = cipher_evp(crypti->passwd, crypti->passwdlen,
 				      orig_message, len, 
 				      type, retlen, ivlen)))
 	    {
@@ -407,32 +420,32 @@ unsigned char *	cipher_message (const unsigned char *orig_message, size_t len, C
 #endif
 	    return ciphertext;
 	}
-	else if (key->type == SEDCRYPT || key->type == SEDSHACRYPT)
+	else if (crypti->sed_type == SEDCRYPT || crypti->sed_type == SEDSHACRYPT)
 	{
 		unsigned char *	ciphertext;
 
 		ciphertext = new_malloc(len + 1);
 		memmove(ciphertext, orig_message, len);
-		encrypt_sed(ciphertext, len, key->key, strlen(key->key));
+		encrypt_sed(ciphertext, len, crypti->passwd, strlen(crypti->passwd));
 		*retlen = len;
 		return ciphertext;
 	}
-	else if (key->type == PROGCRYPT)
+	else if (crypti->sed_type == PROGCRYPT)
 	{
 		unsigned char *ciphertext;
 
-		ciphertext = encrypt_by_prog(orig_message, &len, key);
+		ciphertext = encrypt_by_prog(orig_message, &len, crypti);
 		*retlen = len;
 		return ciphertext;
 	}
 	else
-		panic(1, "cipher_message: key->type %d is not valid", key->type);
+		panic(1, "cipher_message: crypti->sed_type %d is not valid", crypti->sed_type);
 
 	return NULL;
 }
 
 #ifdef HAVE_SSL
-static char *	cipher_evp (const unsigned char *key, int keylen, const unsigned char *plaintext, int plaintextlen, const EVP_CIPHER *type, int *retsize, int ivsize)
+static char *	cipher_evp (const unsigned char *passwd, int passwdlen, const unsigned char *plaintext, int plaintextlen, const EVP_CIPHER *type, int *retsize, int ivsize)
 {
         unsigned char *outbuf;
         int     outlen = 0;
@@ -452,8 +465,8 @@ static char *	cipher_evp (const unsigned char *key, int keylen, const unsigned c
 	{
 	    if (ivsize % sizeof(u_32int_t) != 0)
 		panic(1, "The IV size for a crypto type you're using is %d "
-			"which is not a multiple of %d", 
-			ivsize, sizeof(u_32int_t));
+			"which is not a multiple of %ld", 
+			ivsize, (long)sizeof(u_32int_t));
 
 	    iv = new_malloc(ivsize);
 	    for (iv_count = 0; iv_count < ivsize; iv_count += sizeof(u_32int_t))
@@ -468,8 +481,8 @@ static char *	cipher_evp (const unsigned char *key, int keylen, const unsigned c
 		memcpy(outbuf, iv, ivsize);
 
         EVP_EncryptInit_ex(&a, type, NULL, NULL, iv);
-	EVP_CIPHER_CTX_set_key_length(&a, keylen);
-        EVP_EncryptInit_ex(&a, NULL, NULL, key, NULL);
+	EVP_CIPHER_CTX_set_key_length(&a, passwdlen);
+        EVP_EncryptInit_ex(&a, NULL, NULL, passwd, NULL);
         EVP_EncryptUpdate(&a, outbuf + ivsize, &outlen, plaintext, plaintextlen);
 	EVP_EncryptFinal_ex(&a, outbuf + ivsize + outlen, &extralen);
         EVP_CIPHER_CTX_cleanup(&a);
@@ -490,47 +503,47 @@ static char *	cipher_evp (const unsigned char *key, int keylen, const unsigned c
 }
 #endif
 
-void     encrypt_sed (unsigned char *str, int len, const unsigned char *key, int key_len)
+void     encrypt_sed (unsigned char *str, int len, const unsigned char *passwd, int passwd_len)
 {
-        int     key_pos,
+        int     passwd_pos,
                 i;
         char    mix, 
                 tmp;
 
-        if (!key)
+        if (!passwd)
                 return;                 /* no encryption */
 
-        key_len = strlen(key);
-        key_pos = 0;
+        passwd_len = strlen(passwd);
+        passwd_pos = 0;
         mix = 0;
 
         for (i = 0; i < len; i++)
         {
                 tmp = str[i];
-                str[i] = mix ^ tmp ^ key[key_pos];
+                str[i] = mix ^ tmp ^ passwd[passwd_pos];
                 mix ^= tmp;
-                key_pos = (key_pos + 1) % key_len;
+                passwd_pos = (passwd_pos + 1) % passwd_len;
         }
         str[i] = (char) 0;
 }
 
-static char *	encrypt_by_prog (const unsigned char *str, size_t *len, Crypt *key)
+static char *	encrypt_by_prog (const unsigned char *str, size_t *len, Crypt *crypti)
 {
         char    *ret = NULL, *input;
         char *  args[3];
         int     iplen;
 
-        args[0] = malloc_strdup(key->prog);
+        args[0] = malloc_strdup(crypti->prog);
         args[1] = malloc_strdup("encrypt");
         args[2] = NULL;
-        input = malloc_strdup2(key->key, "\n");
+        input = malloc_strdup2(crypti->passwd, "\n");
 
         iplen = strlen(input);
         new_realloc((void**)&input, *len + iplen);
         memmove(input + iplen, str, *len);
 
         *len += iplen;
-        ret = exec_pipe(key->prog, input, len, args);
+        ret = exec_pipe(crypti->prog, input, len, args);
 
         new_free(&args[0]);
         new_free(&args[1]);
@@ -543,7 +556,7 @@ static char *	encrypt_by_prog (const unsigned char *str, size_t *len, Crypt *key
 
 /**************************************************************************/
 #ifdef HAVE_SSL
-static void	ext256_key (const char *orig, size_t orig_len, char **key, size_t *keylen)
+static void	ext256_passwd (const char *orig, size_t orig_len, char **passwd, size_t *passwdlen)
 {
 	size_t	len;
 
@@ -552,37 +565,48 @@ static void	ext256_key (const char *orig, size_t orig_len, char **key, size_t *k
 	else
 		len = 32;
 
-	*key = new_malloc(32);
-	memset(*key, 0, 32);
-	memcpy(*key, orig, len);
-	*keylen = 32;
+	*passwd = new_malloc(32);
+	memset(*passwd, 0, 32);
+	memcpy(*passwd, orig, len);
+	*passwdlen = 32;
 }
 
-static void	sha256_key (const char *orig, size_t orig_len, char **key, size_t *keylen)
+static void	sha256_passwd (const char *orig, size_t orig_len, char **passwd, size_t *passwdlen)
 {
-	*key = new_malloc(32);
-	sha256(orig, orig_len, *key);
-	*keylen = 32;
+	*passwd = new_malloc(32);
+	sha256(orig, orig_len, *passwd);
+	*passwdlen = 32;
 }
 
-static void	copy_key (const char *orig, size_t orig_len, char **key, size_t *keylen)
+static void	copy_passwd (const char *orig, size_t orig_len, char **passwd, size_t *passwdlen)
 {
-	*key = malloc_strdup(orig);
-	*keylen = orig_len;
+#if 0
+	size_t	passwd_len;
+	orig_len = strlen(orig);		/* XXX for now */
+
+	if ((passwd_len = orig_len) < 9)
+		passwd_len = 9;
+
+	*passwd = new_malloc(passwd_len);
+	memset(*passwd, 0, passwd_len);
+	strlcpy(*passwd, orig, passwd_len);
+#else
+	*passwd = malloc_strdup(orig);
+#endif
+	*passwdlen = orig_len;
 }
 
 /*
  * These are helper functions for $xform() to do SSL strong crypto.
  * XXX These are cut and pasted from decipher_message. 
  */
-#define CRYPTO_HELPER_FUNCTIONS(x, y, blocksize, ivsize, make_key, trim) \
+#define CRYPTO_HELPER_FUNCTIONS(x, y, blocksize, ivsize, make_passwd, trim) \
 ssize_t	x ## _encoder (const char *orig, size_t orig_len, const void *meta, size_t meta_len, char *dest, size_t dest_len) \
 { 									\
-	size_t	len; 							\
 	int	retsize = 0; 						\
 	char *	retval; 						\
-	char *	realkey;						\
-	size_t	realkeylen;						\
+	char *	realpasswd;						\
+	size_t	realpasswdlen;						\
 									\
 	if (orig_len == 0) 						\
 	{ 								\
@@ -590,8 +614,8 @@ ssize_t	x ## _encoder (const char *orig, size_t orig_len, const void *meta, size
 		return 0; 						\
 	} 								\
 									\
-	make_key (meta, meta_len, &realkey, &realkeylen);		\
-	retval = cipher_evp(realkey, realkeylen, orig, orig_len,	\
+	make_passwd (meta, meta_len, &realpasswd, &realpasswdlen);	\
+	retval = cipher_evp(realpasswd, realpasswdlen, orig, orig_len,	\
 				y (), &retsize, ivsize); 		\
 	if (retval && retsize > 0) 					\
 	{ 								\
@@ -608,7 +632,7 @@ ssize_t	x ## _encoder (const char *orig, size_t orig_len, const void *meta, size
 									\
 	if (retval) 							\
 		new_free(&retval); 					\
-	new_free(&realkey);						\
+	new_free(&realpasswd);						\
 	return 0; 							\
 }									\
 									\
@@ -617,8 +641,8 @@ ssize_t	x ## _decoder (const char *ciphertext, size_t len, const void *meta, siz
 	unsigned char *	outbuf = NULL; 					\
 	int	bytes_to_trim = 0;					\
 	int 	retlen = 0; 						\
-	char *	realkey;						\
-	size_t	realkeylen;						\
+	char *	realpasswd;						\
+	size_t	realpasswdlen;						\
 									\
 	if (len == 0)							\
 	{								\
@@ -626,8 +650,8 @@ ssize_t	x ## _decoder (const char *ciphertext, size_t len, const void *meta, siz
 		return 0; 						\
 	}								\
 									\
-	make_key (meta, meta_len, &realkey, &realkeylen);		\
-	if (!(outbuf = decipher_evp(realkey, realkeylen, ciphertext, len, \
+	make_passwd (meta, meta_len, &realpasswd, &realpasswdlen);	\
+	if (!(outbuf = decipher_evp(realpasswd, realpasswdlen, ciphertext, len, \
 				y (), &retlen, ivsize))) 		\
 	{ 								\
 		yell("bummer"); 					\
@@ -636,7 +660,13 @@ ssize_t	x ## _decoder (const char *ciphertext, size_t len, const void *meta, siz
 									\
 	if ( trim )							\
 	{								\
-		bytes_to_trim = outbuf[len - 1] & (blocksize - 1); 	\
+		bytes_to_trim = outbuf[len - 1]; 			\
+		if (bytes_to_trim > blocksize)				\
+		{							\
+			yell("Wants to trim %d byts with block size of %d",\
+				bytes_to_trim, blocksize);		\
+			bytes_to_trim = blocksize - 1;			\
+		}							\
 		outbuf[len - bytes_to_trim] = 0;  			\
 		memmove(outbuf, outbuf + ivsize, len - bytes_to_trim); 	\
 	}								\
@@ -647,11 +677,11 @@ ssize_t	x ## _decoder (const char *ciphertext, size_t len, const void *meta, siz
 	return retlen;							\
 }
 
-CRYPTO_HELPER_FUNCTIONS(blowfish, EVP_bf_cbc, 8, 8, copy_key, 1)
-CRYPTO_HELPER_FUNCTIONS(fish, EVP_bf_ecb, 8, 0, copy_key, 0)
-CRYPTO_HELPER_FUNCTIONS(cast5, EVP_cast5_cbc, 8, 8, copy_key, 1)
-CRYPTO_HELPER_FUNCTIONS(aes, EVP_aes_256_cbc, 16, 16, ext256_key, 1)
-CRYPTO_HELPER_FUNCTIONS(aessha, EVP_aes_256_cbc, 16, 16, sha256_key, 1)
+CRYPTO_HELPER_FUNCTIONS(blowfish, EVP_bf_cbc, 8, 8, copy_passwd, 1)
+CRYPTO_HELPER_FUNCTIONS(fish, EVP_bf_ecb, 8, 0, copy_passwd, 1)
+CRYPTO_HELPER_FUNCTIONS(cast5, EVP_cast5_cbc, 8, 8, copy_passwd, 1)
+CRYPTO_HELPER_FUNCTIONS(aes, EVP_aes_256_cbc, 16, 16, ext256_passwd, 1)
+CRYPTO_HELPER_FUNCTIONS(aessha, EVP_aes_256_cbc, 16, 16, sha256_passwd, 1)
 #endif
 
 
