@@ -1,11 +1,10 @@
-/* $EPIC: exec.c,v 1.27 2006/10/19 22:25:29 jnelson Exp $ */
 /*
  * exec.c: handles exec'd process for IRCII 
  *
  * Copyright (c) 1990 Michael Sandroff.
  * Copyright (c) 1991, 1992 Troy Rollo.
  * Copyright (c) 1992-1996 Matthew Green.
- * Copyright  1997, 2003 EPIC Software Labs
+ * Copyright 1997, 2014 EPIC Software Labs
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -49,7 +48,27 @@
 #include "output.h"
 #include "parse.h"
 #include "newio.h"
-#include "if.h"
+#include "ifcmd.h"
+
+#ifdef NO_JOB_CONTROL
+BUILT_IN_COMMAND(execcmd)
+{ 
+	say("Your system does not support job control, sorry.");
+	return; 
+}
+int	get_child_exit (pid_t x)			{ return -1; }
+void	clean_up_processes (void)			{ return; }
+int	text_to_process (int x, const char *y, int z) 	
+{ 
+	say("Cannot send text to process without job control, sorry.");
+	return 1; 
+}
+void	exec_server_delete (int x)			{ return; }
+void	add_process_wait (int x, const char *y)		{ return; }
+int	get_process_index (char **x)			{ return -1; }
+int	is_valid_process (const char *x)		{ return -1; }
+int	process_is_running (char *x)			{ return 0; }
+#else
 
 #include <sys/wait.h>
 #include <sys/ioctl.h>
@@ -93,21 +112,15 @@ static	int	process_list_size = 0;
 static 	void 	handle_filedesc 	(Process *, int *, int, int);
 static 	void 	cleanup_dead_processes 	(void);
 static 	void 	ignore_process 		(int);
-static 	void 	close_in		(int);
+static 	void 	exec_close_in		(int);
+static 	void 	exec_close_out		(int);
 static 	void 	kill_process 		(int, int);
 static 	void 	kill_all_processes 	(int signo);
 static 	int 	valid_process_index 	(int proccess);
 static 	int 	is_logical_unique 	(char *logical);
 static	int 	logical_to_index 	(const char *logical);
-
-/*
- * A nice array of the possible signals.  Used by the coredump trapping
- * routines and in the exec.c package 
- */
-#ifndef SYS_SIGLIST_DECLARED
-#include "sig.inc"
-#endif
-
+static	int	index_to_target 	(int, char *, size_t);
+static	void 	do_exec 		(int fd);
 
 /*
  * exec: the /EXEC command.  Does the whole shebang for ircII sub procceses
@@ -118,8 +131,8 @@ static	int 	logical_to_index 	(const char *logical);
 BUILT_IN_COMMAND(execcmd)
 {
 	const char	*who = NULL;
-	char		*logical = NULL,
-			*flag;
+	char		*logical = NULL;
+	const char	*flag;
 	const char 	*redirect = NULL;
 	unsigned 	refnum = 0;
 	size_t		len;
@@ -181,9 +194,9 @@ BUILT_IN_COMMAND(execcmd)
 			else
 				redirect = "PRIVMSG";
 
-			if (!(who = get_echannel_by_refnum(0)))
+			if (!(who = get_target_by_refnum(0)))
 			{
-			     say("No current channel in this window for -OUT");
+			     say("No query or channel in this window for -OUT");
 			     return;
 			}
 		}
@@ -297,7 +310,20 @@ BUILT_IN_COMMAND(execcmd)
 			if ((i = get_process_index(&args)) == -1)
 				return;
 
-			close_in(i);
+			exec_close_in(i);
+			return;
+		}
+
+		/*
+		 * /EXEC -CLOSEIN close the processes STDIN,
+		 * in the hope that it will take the hint.
+		 */
+		else if (my_strnicmp(flag, "CLOSEOUT", len) == 0)
+		{
+			if ((i = get_process_index(&args)) == -1)
+				return;
+
+			exec_close_out(i);
 			return;
 		}
 
@@ -368,9 +394,9 @@ BUILT_IN_COMMAND(execcmd)
 			 */
 			for (sig = 1; sig < NSIG; sig++)
 			{
-				if (!sys_siglist[sig])
+				if (!get_signal_name(sig))
 					continue;
-				if (!my_strnicmp(sys_siglist[sig], flag, len))
+				if (!my_strnicmp(get_signal_name(sig), flag, len))
 				{
 					kill_process(i, sig);
 					return;
@@ -394,6 +420,8 @@ BUILT_IN_COMMAND(execcmd)
 	 */
 	if (*args == '%')
 	{
+		int	l;
+
 		/*
 		 * Make sure the process is actually running
 		 */
@@ -401,7 +429,7 @@ BUILT_IN_COMMAND(execcmd)
 			return;
 
 		proc = process_list[i];
-		message_to(refnum);
+		l = message_setall(refnum, NULL, LEVEL_OTHER);
 
 		/*
 		 * Check to see if the user wants to change windows
@@ -453,7 +481,7 @@ say("Output from process %d (%s) now going to you", i, proc->name);
 		if (stderrpc)
 			malloc_strcpy(&proc->stderrpc, stderrpc);
 
-		message_to(-1);
+		pop_message_from(l);
 	}
 
 	/*
@@ -465,7 +493,9 @@ say("Output from process %d (%s) now going to you", i, proc->name);
 			pid, cnt;
 		char	*shell,
 			*arg;
-		char	*name = args;
+		char	*name;
+
+		name = LOCAL_COPY(args);
 
 		if (!is_logical_unique(logical))
 		{
@@ -482,12 +512,12 @@ say("Output from process %d (%s) now going to you", i, proc->name);
 		if (pipe(p0) || pipe(p1) || pipe(p2))
 		{
 			say("Unable to start new process: %s", strerror(errno));
-			new_close(p0[0]);
-			new_close(p0[1]);
-			new_close(p1[0]);
-			new_close(p1[1]);
-			new_close(p2[0]);
-			new_close(p2[1]);
+			close(p0[0]);
+			close(p0[1]);
+			close(p1[0]);
+			close(p1[1]);
+			close(p2[0]);
+			close(p2[1]);
 			return;
 		}
 
@@ -495,6 +525,12 @@ say("Output from process %d (%s) now going to you", i, proc->name);
 		{
 		case -1:
 			say("Couldn't start new process!");
+			close(p0[0]);
+			close(p0[1]);
+			close(p1[0]);
+			close(p1[1]);
+			close(p2[0]);
+			close(p2[1]);
 			break;
 
 		/*
@@ -507,8 +543,10 @@ say("Output from process %d (%s) now going to you", i, proc->name);
 			 * Sever all ties we had with the parent ircII process
 			 */
 			setsid();
-			setuid(getuid());
-			setgid(getgid());
+			if (setuid(getuid()))
+				_exit(0);
+			if (setgid(getgid()))
+				_exit(0);
 			my_signal(SIGINT, SIG_IGN);
 			my_signal(SIGQUIT, SIG_DFL);
 			my_signal(SIGSEGV, SIG_DFL);
@@ -569,7 +607,7 @@ say("Output from process %d (%s) now going to you", i, proc->name);
 
 				execl(shell, shell, flag, name, NULL);
 
-				printf("*** Error running program \"%s %s\": %s\n",
+				printf("*** Error running program \"%s %s\": %s\n", 
 						shell, args, strerror(errno));
 			}
 
@@ -586,9 +624,9 @@ say("Output from process %d (%s) now going to you", i, proc->name);
 		default:
 		{
 			proc = new_malloc(sizeof(Process));
-			new_close(p0[0]);
-			new_close(p1[1]);
-			new_close(p2[1]);
+			close(p0[0]);
+			close(p1[1]);
+			close(p2[1]);
 
 			/*
 			 * Init the proc list if neccesary
@@ -674,8 +712,8 @@ say("Output from process %d (%s) now going to you", i, proc->name);
 			if (endc)
 				add_process_wait(proc->index, endc);
 
-			new_open(proc->p_stdout);
-			new_open(proc->p_stderr);
+			new_open(proc->p_stdout, do_exec, NEWIO_READ, 1, proc->server);
+			new_open(proc->p_stderr, do_exec, NEWIO_READ, 1, proc->server);
 			break;
 		}
 		}
@@ -691,7 +729,7 @@ say("Output from process %d (%s) now going to you", i, proc->name);
  * are closed.  If EOF has been asserted on both, then  we mark the process
  * as being "dumb".  Once it is reaped (exited), it is expunged.
  */
-void 		do_processes (fd_set *rd, fd_set *wd)
+void 		do_exec (int fd)
 {
 	int	i;
 	int	limit;
@@ -707,16 +745,14 @@ void 		do_processes (fd_set *rd, fd_set *wd)
 		if (!proc)
 			continue;
 
-		if (proc->p_stdout != -1 && FD_ISSET(proc->p_stdout, rd))
+		if (proc->p_stdout != -1 && proc->p_stdout == fd)
 		{
-			FD_CLR(proc->p_stdout, rd);
 			handle_filedesc(proc, &proc->p_stdout, 
 					EXEC_PROMPT_LIST, EXEC_LIST);
 		}
 
-		if (proc->p_stderr != -1 && FD_ISSET(proc->p_stderr, rd))
+		if (proc->p_stderr != -1 && proc->p_stderr == fd)
 		{
-			FD_CLR(proc->p_stderr, rd);
 			handle_filedesc(proc, &proc->p_stderr,
 					EXEC_PROMPT_LIST, EXEC_ERRORS_LIST);
 		}
@@ -736,92 +772,109 @@ static void 	handle_filedesc (Process *proc, int *fd, int hook_nonl, int hook_nl
 {
 	char 	exec_buffer[IO_BUFFER_SIZE + 1];
 	ssize_t	len;
+	int	ofs;
+	const char *callback = NULL;
+	int	hook = -1;
+	int	l;
+	char	logical_name[1024];
+	const char	*utf8_text;
+	char *extra = NULL;
 
-	switch ((len = dgets(*fd, exec_buffer, IO_BUFFER_SIZE, 0, NULL))) /* No buffering! */
+	/* No buffering! */
+	switch ((len = dgets(*fd, exec_buffer, IO_BUFFER_SIZE, 0))) 
 	{
-		case -1:	/* Something died */
+	    case -1:		/* Something died */
+	    {
+		*fd = new_close(*fd);
+		if (proc->p_stdout == -1 && proc->p_stderr == -1)
+			proc->dumb = 1;
+		return;				/* PUNT! */
+	    }
+
+	    case 0:		/* We didnt get a full line */
+	    {
+		/* 
+		 * XXX This is a hack.  dgets() can return 0 for a line
+		 * containing solely a newline, as well as a line that didn't
+		 * have a newline.  So we have to check to see if the line 
+		 * contains only a newline!
+		 */
+		if (exec_buffer[0] != '\n')
 		{
-			*fd = new_close(*fd);
-			if (proc->p_stdout == -1 && proc->p_stderr == -1)
-				proc->dumb = 1;
-			break;
+		    if (hook_nl == EXEC_LIST)
+		    {
+			if (proc->stdoutpc && *proc->stdoutpc)
+			    callback = proc->stdoutpc;
+		    }
+		    else if (hook_nl == EXEC_ERRORS_LIST)
+		    {
+			if (proc->stderrpc && *proc->stderrpc)
+			    callback = proc->stderrpc;
+		    }
+		    hook = hook_nonl;
+		    break;
 		}
-		case 0:		/* We didnt get a full line */
+
+		/* XXX HACK -- Line contains only a newline.  */
+		*exec_buffer = 0;
+		/* FALLTHROUGH */
+	    }
+
+	    default:		/* We got a full line */
+	    {
+		if (hook_nl == EXEC_LIST)
 		{
-			/* 
-			 * XXX XXX Major, world class hack here XXX XXX 
-			 * Since this problem has been addressed already
-			 * by newio2.c, it doesn't matter if we just fix the
-			 * symptom here since this code is all obsolete.
-			 */
-			if (exec_buffer[0] == '\n')
-			{
-				exec_buffer[0] = 0;
-				goto this_sucks;
-			}
-
-			     if (hook_nl == EXEC_LIST && proc->stdoutpc && *proc->stdoutpc)
-				parse_line("EXEC", proc->stdoutpc, 
-						exec_buffer, 0, 0);
-			else if (hook_nl == EXEC_ERRORS_LIST && proc->stderrpc && *proc->stderrpc)
-				parse_line("EXEC", proc->stderrpc, 
-						exec_buffer, 0, 0);
-			else if (proc->logical)
-				do_hook(hook_nonl, "%s %s", 
-					proc->logical, exec_buffer);
-			else
-				do_hook(hook_nonl, "%d %s", 
-					proc->index, exec_buffer);
-
-			set_prompt_by_refnum(proc->refnum, exec_buffer);
-			break;
+		    if (proc->stdoutc && *proc->stdoutc)
+			callback = proc->stdoutc;
 		}
-		default:	/* We got a full line */
-this_sucks:
+		else if (hook_nl == EXEC_ERRORS_LIST)
 		{
-			int ofs;
-
-			ofs = from_server;
-			from_server = proc->server;
-			if (proc->refnum)
-				message_to(proc->refnum);
-			proc->counter++;
-
-			while (len > 0 && (exec_buffer[len] == '\n' ||
-					   exec_buffer[len] == '\r'))
-			{
-				exec_buffer[len--] = 0;
-			}
-
-			if (proc->redirect) 
-				redirect_text(proc->server, proc->who, 
-					exec_buffer, proc->redirect, 1);
-
-			if (hook_nl == EXEC_LIST && proc->stdoutc && *proc->stdoutc)
-				parse_line("EXEC", proc->stdoutc, 
-						exec_buffer, 0, 0);
-			else if (hook_nl == EXEC_ERRORS_LIST && proc->stderrc && *proc->stderrc)
-				parse_line("EXEC", proc->stderrc, 
-						exec_buffer, 0, 0);
-			else if (proc->logical)
-			{
-				if ((do_hook(hook_nl, "%s %s", 
-						proc->logical, exec_buffer)))
-					if (!proc->redirect)
-						put_it("%s", exec_buffer);
-			}
-			else
-			{
-				if ((do_hook(hook_nl, "%d %s", 
-						proc->index, exec_buffer)))
-					if (!proc->redirect)
-						put_it("%s", exec_buffer);
-			}
-
-			message_to(-1);
-			from_server = ofs;
+		    if (proc->stderrc && *proc->stderrc)
+			callback = proc->stderrc;
 		}
+		hook = hook_nl;
+		break;
+	    }
 	}
+
+	ofs = from_server;
+	from_server = proc->server;
+	if (proc->refnum)
+		l = message_setall(proc->refnum, NULL, LEVEL_OTHER);
+	else
+		l = message_from(NULL, LEVEL_OTHER);
+
+	proc->counter++;
+
+	while (len > 0 && (exec_buffer[len - 1] == '\n' ||
+			   exec_buffer[len - 1] == '\r'))
+	     exec_buffer[--len] = 0;
+
+	index_to_target(proc->index, logical_name, sizeof(logical_name));
+	utf8_text = inbound_recode(logical_name, proc->server, empty_string, exec_buffer, &extra);
+
+	if (proc->redirect) 
+	     redirect_text(proc->server, proc->who, 
+				utf8_text, proc->redirect, 1);
+
+	if (callback)
+	    call_lambda_command("EXEC", callback, utf8_text);
+	else if (proc->logical)
+	{
+	     if ((do_hook(hook, "%s %s", proc->logical, utf8_text)))
+		if (!proc->redirect)
+		    put_it("%s", utf8_text);
+	}
+	else
+	{
+	    if ((do_hook(hook, "%d %s", proc->index, utf8_text)))
+		if (!proc->redirect)
+		    put_it("%s", utf8_text);
+	}
+
+	new_free(&extra);
+	pop_message_from(l);
+	from_server = ofs;
 }
 
 /*
@@ -861,6 +914,13 @@ int 		get_child_exit (pid_t wanted)
 		 */
 		if (wanted != -1 && pid == wanted)
 		{
+			/* 
+			 * We do not clear 'dead_children_processes' here
+			 * because we do not know if we've reaped all of
+			 * the children yet!  Leaving it set means this 
+			 * function is called again, and then if there are
+			 * no more left, it is cleared (below).
+			 */
 		        unblock_signal(SIGCHLD);
 
 			if (WIFEXITED(status))
@@ -938,26 +998,39 @@ void 		clean_up_processes (void)
  */
 int 		text_to_process (int proc_index, const char *text, int show)
 {
-	Process	*	proc;
-	char	*	my_buffer;
+	Process	*proc;
+	char *	my_buffer;
 	size_t	size;
+	char	logical_name[1024];
+	const char *recoded_text;
+	char *	extra = NULL;
 
 	if (valid_process_index(proc_index) == 0)
 		return 1;
 
 	proc = process_list[proc_index];
 
-	message_to(proc->refnum);
 	if (show)
+	{
+		int	l = message_setall(proc->refnum, NULL, LEVEL_OTHER);
 		put_it("%s%s", get_prompt_by_refnum(proc->refnum), text);
-	message_to(-1);
+		pop_message_from(l);
+	}
 
 	size = strlen(text) + 2;
 	my_buffer = alloca(size);
 	snprintf(my_buffer, size, "%s\n", text);
-	write(proc->p_stdin, my_buffer, strlen(my_buffer));
-	set_prompt_by_refnum(proc->refnum, empty_string);
 
+	index_to_target(proc_index, logical_name, sizeof(logical_name));
+	recoded_text = outbound_recode(logical_name, proc->server, my_buffer, &extra);
+	if (write(proc->p_stdin, recoded_text, strlen(recoded_text)) <= 0)
+	{
+		yell("Was unable to write text %s to process %d",
+			text, proc_index);
+	}
+	new_free(&extra);
+
+	set_prompt_by_refnum(proc->refnum, empty_string);
 	return (0);
 }
 
@@ -1022,10 +1095,12 @@ static void 	cleanup_dead_processes (void)
 		*next;
 	Process *deadproc, *proc;
 	char	*exit_info;
+	int	old_from_server, l;
 
 	if (!process_list)
 		return;		/* Nothing to do */
 
+	old_from_server = from_server;
 	for (i = 0; i < process_list_size; i++)
 	{
 		if (!(proc = process_list[i]))
@@ -1061,6 +1136,8 @@ static void 	cleanup_dead_processes (void)
 				deadproc->index, deadproc->termsig, deadproc->retcode);
 		}
 
+		from_server = deadproc->server;
+		l = message_from(NULL, LEVEL_OTHER);
 
 		/*
 		 * First thing we do is run any /wait %proc -cmd commands
@@ -1070,7 +1147,7 @@ static void 	cleanup_dead_processes (void)
 		while ((cmd = next))
 		{
 			next = cmd->next;
-			parse_line(NULL, cmd->name, exit_info, 0, 0);
+			call_lambda_command("WAITPROC", cmd->name, exit_info);
 			new_free(&cmd->name);
 			new_free((char **)&cmd);
 		}
@@ -1080,23 +1157,36 @@ static void 	cleanup_dead_processes (void)
 		 */
 		if (do_hook(EXEC_EXIT_LIST, "%s", exit_info))
 		{
-			if (get_int_var(NOTIFY_ON_TERMINATION_VAR))
+		    if (get_int_var(NOTIFY_ON_TERMINATION_VAR))
+		    {
+			if (deadproc->termsig > 0 && deadproc->termsig < NSIG)
 			{
-				if (deadproc->termsig > 0 && deadproc->termsig < NSIG)
-	say("Process %d (%s) terminated with signal %s (%d)", 
-	   deadproc->index, deadproc->name, sys_siglist[deadproc->termsig], deadproc->termsig);
-				else if (deadproc->disowned)
-	say("Process %d (%s) disowned", deadproc->index, deadproc->name);
-				else
-	say("Process %d (%s) terminated with return code %d", 
-		deadproc->index, deadproc->name, deadproc->retcode);
-			}
-		}
+				say("Process %d (%s) terminated "
+					"with signal %s (%d)", 
+				   deadproc->index, deadproc->name, 
+				   get_signal_name(deadproc->termsig),
+				   deadproc->termsig);
 
-		if (deadproc->p_stdin != -1)
-			new_close(deadproc->p_stdin);
-		new_close(deadproc->p_stdout);
-		new_close(deadproc->p_stderr);
+			}
+			else if (deadproc->disowned)
+			{
+				say("Process %d (%s) disowned", 
+				   deadproc->index, deadproc->name);
+			}
+			else
+			{
+				say("Process %d (%s) terminated "
+					"with return code %d", 
+				   deadproc->index, deadproc->name, 
+				   deadproc->retcode);
+			}
+		    }
+		}
+		pop_message_from(l);
+
+		deadproc->p_stdin = new_close(deadproc->p_stdin);
+		deadproc->p_stdout = new_close(deadproc->p_stdout);
+		deadproc->p_stderr = new_close(deadproc->p_stderr);
 		new_free(&deadproc->name);
 		new_free(&deadproc->logical);
 		new_free(&deadproc->who);
@@ -1122,6 +1212,8 @@ static void 	cleanup_dead_processes (void)
 		process_list_size = i + 1;
 		RESIZE(process_list, Process, process_list_size);
 	}
+
+	from_server = old_from_server;
 }
 
 
@@ -1135,20 +1227,8 @@ static void 	cleanup_dead_processes (void)
  */
 static void 	ignore_process (int idx)
 {
-	Process *proc;
-
-	if (valid_process_index(idx) == 0)
-		return;
-
-	proc = process_list[idx];
-	if (proc->p_stdin != -1)
-		proc->p_stdin = new_close(proc->p_stdin);
-	if (proc->p_stdout != -1)
-		proc->p_stdout = new_close(proc->p_stdout);
-	if (proc->p_stderr != -1)
-		proc->p_stderr = new_close(proc->p_stderr);
-
-	proc->dumb = 1;
+	exec_close_in(idx);
+	exec_close_out(idx);
 }
 
 /*
@@ -1156,7 +1236,7 @@ static void 	ignore_process (int idx)
  * rest of its output, we close its input, and hopefully it will get the
  * message and close up shop.
  */
-static void 	close_in (int idx)
+static void 	exec_close_in (int idx)
 {
 	Process *proc;
 
@@ -1168,7 +1248,26 @@ static void 	close_in (int idx)
 		proc->p_stdin = new_close(proc->p_stdin);
 }
 
+/*
+ * close_out:  When we are done sending to a process sometimes we have to 
+ * close our stdout before they will do their thing and send us data back
+ * to stdin.  
+ */
+static void 	exec_close_out (int idx)
+{
+	Process *proc;
 
+	if (valid_process_index(idx) == 0)
+		return;
+
+	proc = process_list[idx];
+	if (proc->p_stdout != -1)
+		proc->p_stdout = new_close(proc->p_stdout);
+	if (proc->p_stderr != -1)
+		proc->p_stderr = new_close(proc->p_stderr);
+
+	proc->dumb = 1;
+}
 
 
 /*
@@ -1180,7 +1279,8 @@ static void 	close_in (int idx)
  */
 static void 	kill_process (int kill_index, int sig)
 {
-	pid_t pgid;
+	pid_t	pgid;
+	int	old_from_server, l;
 
 	if (!process_list || kill_index > process_list_size || 
 			!process_list[kill_index])
@@ -1189,10 +1289,17 @@ static void 	kill_process (int kill_index, int sig)
 		return;
 	}
 
+	old_from_server = from_server;
+	from_server = process_list[kill_index]->server;
+	l = message_from(NULL, LEVEL_OTHER);
+
 	say("Sending signal %s (%d) to process %d: %s", 
-		sys_siglist[sig], sig, kill_index, 
+		get_signal_name(sig), sig, kill_index, 
 		process_list[kill_index]->name);
 
+	pop_message_from(l);
+	from_server = old_from_server;
+
 #ifdef HAVE_GETPGID
 	pgid = getpgid(process_list[kill_index]->pid);
 #else
@@ -1317,6 +1424,24 @@ static	int 	logical_to_index (const char *logical)
 	return -1;
 }
 
+static	int	index_to_target (int process, char *buf, size_t bufsiz)
+{
+	if ((process < 0) || (process >= process_list_size) ||
+			!process_list[process])
+	{
+		snprintf(buf, bufsiz, "(Process %d does not exist)", process);
+		return -1;
+	}
+
+	/* /EXEC targets are always preceded by a percent sign */
+	if (process_list[process]->logical)
+		snprintf(buf, bufsiz, "%%%s", process_list[process]->logical);
+	else 
+		snprintf(buf, bufsiz, "%%%d", process);
+
+	return 0;
+}
+
 /*
  * get_process_index: parses out a process index or logical name from the
  * given string 
@@ -1354,3 +1479,5 @@ int		process_is_running (char *arg)
 	else
 		return 1;
 }
+
+#endif
