@@ -1,8 +1,7 @@
-/* $EPIC: files.c,v 1.43 2012/03/25 02:08:06 jnelson Exp $ */
 /*
  * files.c -- allows you to read/write files. Wow.
  *
- * Copyright  1995, 2003 EPIC Software Labs
+ * Copyright 1995, 2003 EPIC Software Labs
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,7 +37,8 @@
 #include "output.h"
 #include "elf.h"
 
-/* Here's the plan.
+/* 
+ * Here's the plan...
  *  You want to open a file.. you can READ it or you can WRITE it.
  *    unix files can be read/written, but its not the way you expect.
  *    so we will only alllow one or the other.  If you try to write to
@@ -70,7 +70,7 @@ typedef struct FILE___ File;
 
 static File *	FtopEntry = (File *) 0;
 
-static File *	new_file (struct epic_loadfile *elf/*FILE *the_file*/)
+static File *	new_file (struct epic_loadfile *elf)
 {
         static long int id = 0;
         File *tmp = FtopEntry;
@@ -109,34 +109,32 @@ static void	remove_file (File *file)
 			tmp->next = tmp->next->next;
 	}
 	epic_fclose(file->elf);
-	new_free((char **)&file);
+	new_free(&file->elf);
+	new_free(&file);
 }
 
 
 int	open_file_for_read (const char *filename)
 {
-	char *dummy_filename = (char *) 0;
-        struct epic_loadfile *elf;
-        struct stat sb;
+	char *dummy_filename = malloc_strdup(filename);
+	struct epic_loadfile *elf;
+	struct stat sb;
+	File *fr;
 
-        File * fr;
-
-	malloc_strcpy(&dummy_filename, filename);
 	elf = uzfopen(&dummy_filename, ".", 1, &sb);
 	new_free(&dummy_filename);
 
-        if (!elf) {
-                new_free(&elf);
-                return -1;
-        }
+	if (!elf)
+		return -1;
 
 	if (sb.st_mode & 0111)
 	{
 		say("Cannot open %s -- executable file", filename);	
+		new_free(&elf);
 		return -1;
 	}
 
-	fr=new_file(elf);
+	fr = new_file(elf);
 	return fr->id;
 }
 
@@ -225,29 +223,6 @@ static File 			retval = {0 , &elf, NULL};
 	return &retval;
 }
 
-/*
- * target_file_write: "Send" a "message" to a file or logfile
- * Examples:
- *	/msg @<fileref> This is logged to an $open() file.
- *	/msg @W<winref> This is logged to a window's logfile
- *	/msg @L<logref> This is logged to a general logfile
- * This function is nothing but a bald wrapper to file_write which does
- * all of the heavy lifting.
- */
-int 	target_file_write (const char *fd, const char *stuff)
-{
-	/* XXX Really we should call add_to_log() here */
-	/* XXX file_write() should not know or care about logfiles */
-	if (toupper(*fd) == 'W' && is_number(fd + 1))
-		return file_write(1, my_atol(fd + 1), stuff);
-	else if (toupper(*fd) == 'L' && is_number(fd + 1))
-		return file_write(2, my_atol(fd + 1), stuff);
-	else if (is_number(fd))
-		return file_write(0, my_atol(fd), stuff);
-	else
-		return -1;
-}
-
 /*
  * file_write: Write something to a file or logfile
  *
@@ -259,7 +234,7 @@ int 	target_file_write (const char *fd, const char *stuff)
  *
  * XXX - I think writes to logfiles should be sent back to the logging funcs
  */
-int	file_write (int logtype, int fd, const char *stuff)
+static int	file_write (int logtype, int fd, const char *stuff)
 {
 	File 	*ptr;
 	int	retval;
@@ -275,12 +250,69 @@ int	file_write (int logtype, int fd, const char *stuff)
 		return -1;
 
 	/* XXX This should call add_to_log  if it's a logfile */
-	retval = fprintf(ptr->elf->fp, "%s\n", stuff);
+	retval = fprintf(ptr->elf->fp, "%s\n", stuff); /* XXX utf8 XXX */
 	if ((fflush(ptr->elf->fp)) == EOF)
 		return -1;
 	return retval;
 }
 
+/*
+ * target_file_write: "Send" a "message" to a file or logfile
+ *
+ * Arguments:
+ * 	fd - A logical target to a logfile: 
+ *	    Syntax: @ + [<domain>] + <number>
+ *	    Specifically:
+ *		@W<winref>	Write to the /WINDOW LOG for <winref>
+ *		@L<logref>	Write to a /LOG file
+ *		@<openref>	Write to an $open() file
+ *	stuff - A UTF8 string to be sent to the log
+ *
+ * Return value:
+ *	-1	The "fd" argument was invalid becuase;
+ *		1. It was not in the form @W<number>, @L<number>
+ *		   or @<number>.
+ *	or, the return value of file_write() above:
+ *		-1 	The "fd" argument was not a valid logfile
+ *			(ie, the window has no log, the /log refnum
+ *			does not exist or is off)
+ *		-1 	The message was not written() becuase 
+ *			fflush() returned EOF
+ *		or, the return value of fprintf().
+ *  
+ * This function is nothing but a bald wrapper to file_write which does
+ * all of the heavy lifting.
+ */
+int 	target_file_write (const char *fd, const char *stuff)
+{
+	if (*fd != '@')
+		return -1;
+
+	fd++;
+	if (*fd == 'W' || *fd == 'w' || *fd == 'L' || *fd == 'l')
+		fd++;
+
+	if (!is_number(fd))
+		return -1;
+
+	/* 
+	 * We specifically do not rewrite the UTF8 string.
+	 * All writes to files are done in UTF8 only.
+	 */
+
+	/* XXX Really we should call add_to_log() here */
+	/* XXX file_write() should not know or care about logfiles */
+	if (toupper(*fd) == 'W' && is_number(fd + 1))
+		return file_write(1, my_atol(fd + 1), stuff);
+	else if (toupper(*fd) == 'L' && is_number(fd + 1))
+		return file_write(2, my_atol(fd + 1), stuff);
+	else if (is_number(fd))
+		return file_write(0, my_atol(fd), stuff);
+	else
+		return -1;
+}
+
+
 /*
  * file_writeb: Write binary data (not a line of text) to a file or logfile
  *
@@ -329,23 +361,23 @@ char *	file_read (int fd)
 	else
 	{
 		char	*ret = NULL;
+		size_t	retlen = 0;
+		size_t	retbufsiz = 0;
 		char	*end = NULL;
-		size_t	len = 0;
-		size_t	newlen = 0;
 
                 if (ptr->elf->fp)
                     clearerr(ptr->elf->fp);
 
 		for (;;)
 		{
-		    newlen += 4096;
-		    RESIZE(ret, char, newlen);
-		    ret[len] = 0;	/* Keep this -- C requires it! */
-		    if (!epic_fgets(ret + len, newlen - len, ptr->elf))
+		    retbufsiz += 4096;
+		    RESIZE(ret, char, retbufsiz);
+		    ret[retlen] = 0;	/* Keep this -- C requires it! */
+		    if (!epic_fgets(ret + retlen, retbufsiz - retlen, ptr->elf))
 			break;
-		    if ((end = strchr(ret + len, '\n')))
+		    if ((end = strchr(ret + retlen, '\n')))
 			break;
-		    len = newlen - 1;
+		    retlen = retbufsiz - 1;
 		}
 
 		/* Do we need to truncate the result? */
@@ -354,6 +386,15 @@ char *	file_read (int fd)
 		else if ( (ptr->elf->fp) && (ferror(ptr->elf->fp)) )
                     *ret = 0;	/* Or the whole thing on error */
 
+		/* XXX TODO -- this is just temporary */
+		if (invalid_utf8str(ret))
+		{
+			const char *encodingx;
+
+			encodingx = find_recoding("scripts", NULL, NULL);
+			recode_with_iconv(encodingx, NULL, &ret, &retlen);
+		}
+
 		return ret;
 	}
 }
@@ -814,3 +855,11 @@ char *	dbmctl (char *input)
 	RETURN_EMPTY;
 }
 
+void	close_all_dbms (void)
+{
+	Dbm *	x;
+
+	while ((x = DtopEntry))
+		close_dbm(x->refnum);
+}
+
