@@ -1,12 +1,11 @@
-/* $EPIC: irc.c,v 1.1325 2012/05/11 01:42:11 jnelson Exp $ */
 /*
  * ircII: a new irc client.  I like it.  I hope you will too!
  *
  * Copyright (c) 1990 Michael Sandroff.
  * Copyright (c) 1991, 1992 Troy Rollo.
  * Copyright (c) 1992-1996 Matthew Green.
- * Copyright  1994 Jake Khuon.
- * Copyright  1993, 2010 EPIC Software Labs.
+ * Copyright 1994 Jake Khuon.
+ * Copyright 1993, 2016 EPIC Software Labs.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,19 +39,19 @@
 /*
  * irc_version is what $J returns, its the common-name for the version.
  */
-const char irc_version[] = "EPIC5-1.1.3";
-const char useful_info[] = "epic5 1 1 3";
+const char irc_version[] = "EPIC5-2.0";
+const char useful_info[] = "epic5 2 0 0";
 
 /*
  * internal_version is what $V returns, its the integer-id for the
  * version, and corresponds to the date of release, YYYYMMDD.
  */ 
-const char internal_version[] = "20120510";
+const char internal_version[] = "20160212";
 
 /*
  * In theory, this number is incremented for every commit.
  */
-const unsigned long	commit_id = 1656;
+const unsigned long	commit_id = 1864;
 
 /*
  * As a way to poke fun at the current rage of naming releases after
@@ -60,7 +59,7 @@ const unsigned long	commit_id = 1656;
  * reality, I have decided to start doing that with EPIC.  These names
  * are intentionally and maliciously silly.  Complaints will be ignored.
  */
-const char ridiculous_version_name[] = "Feculance";
+const char ridiculous_version_name[] = "Gamboling";
 
 #define __need_putchar_x__
 #include "status.h"
@@ -89,6 +88,7 @@ const char ridiculous_version_name[] = "Feculance";
 #include "parse.h"
 #include "levels.h"
 #include "extlang.h"
+#include "files.h"
 #include <pwd.h>
 
 
@@ -163,7 +163,7 @@ int		do_window_notifies = 1;
 /*
  * If set, outbound connections will be bind()ed to the address
  * specified.  if unset, the default address for your host will
- * be used.  LocalHostName can be set by the /HOSTNAME command 
+ * be used.  VHosts can be set by the /HOSTNAME command 
  * or via the IRCHOST environment variable.  These variables should
  * be considered read-only.  Dont ever change them.
  *
@@ -173,7 +173,6 @@ int		do_window_notifies = 1;
  */
 char *		LocalIPv4HostName = NULL;
 char *		LocalIPv6HostName = NULL;
-char *		LocalHostName = NULL;
 
 int		inbound_line_mangler = 0,
 		outbound_line_mangler = 0;
@@ -293,6 +292,7 @@ void	irc_exit (int really_quit, const char *format, ...)
 	logger(&value);
 	get_child_exit(-1);  /* In case some children died in the exit hook. */
 	clean_up_processes();
+	close_all_dbms();
 
 	/* Arrange to have the cursor on the input line after exit */
 	if (!dumb_mode)
@@ -311,12 +311,11 @@ void	irc_exit (int really_quit, const char *format, ...)
 	delete_all_windows();
 	destroy_server_list();
 
-	destroy_call_stack();
 	remove_bindings();
 	flush_on_hooks();
 	flush_all_symbols();
 	window_display = old_window_display;
-	fprintf(stdout, "\r");
+	printf("\r");
 	fflush(stdout);
 
 	if (really_quit)
@@ -393,7 +392,7 @@ The program will now terminate.						\n", irc_version, commit_id, internal_versi
  * the right response to our question.  If the response was affirmative, the
  * user gets booted from irc.  Otherwise, life goes on. 
  */
-static void quit_response (char *dummy, char *ptr)
+static void	quit_response (char *dummy, const char *ptr)
 {
 	int	len;
 
@@ -479,8 +478,8 @@ static	void	parse_args (int argc, char **argv)
 	char *ptr = (char *) 0;
 	char *tmp_hostname = NULL;
 	char *the_path = NULL;
-	char *translation_path = NULL;
-	
+	int size;
+
 	int altargc = 0;
 	char **altargv;
 
@@ -610,61 +609,51 @@ static	void	parse_args (int argc, char **argv)
 	if ((ptr = getenv("IRCHOST")) && *ptr)
 		tmp_hostname = ptr;
 
-	if ((ptr = getenv("IRCTRANSLATIONPATH")))
-		translation_path = malloc_strdup(ptr);
-	else
-		translation_path = malloc_strdup2(IRCLIB, "/translation/");
-
-	set_var_value(TRANSLATION_PATH_VAR, translation_path, 0);
-	new_free(&translation_path);
-	
-
 	/* The -S-option  / shebang tokeniser */
 	if (argc > 1 && ((argv[1][0] == '-' && argv[1][1] == 'S')
-		|| strchr(argv[1], ' ') != NULL 
-		|| strchr(argv[1], '\t') != NULL)
-	)
+			|| strchr(argv[1], ' ') != NULL 
+			|| strchr(argv[1], '\t') != NULL))
 	{
-		int argn, argpos, bufpos, c = 0, n;
+		int 	argn, 
+			i,
+			j,
+			c = 0, 
+			n;
+		size_t	len;
+
 		altargv = new_malloc(sizeof(char *));
-		altargv[0] = new_malloc(strlen(argv[0]) +1);
-		strcpy(altargv[0], argv[0]);
+		altargv[0] = malloc_strdup(argv[0]);
 		altargc = 1;
 		
 		for (argn = 1; argn < argc; argn++)
 		{
-		    int arglen;
-		    char *buffer;
+		    const char *s;
+		    char *	dest;
+		    size_t	destlen;
+		    char *	d;
 
-		    arglen =  strlen(argv[argn]);
-		    buffer = alloca(arglen + 1);
-		    argpos = -1;
-		    do
+		    s = argv[argn];
+		    destlen = strlen(s) + 1;
+		    d = dest = alloca(destlen + 1);
+
+		    while (*s && ((ssize_t)(d - dest) < (ssize_t)destlen))
 		    {
-		 	for (n = 0; n <= arglen; ) buffer[n++] = '\0';
-			bufpos = 0;
-			for (argpos++; argpos <= arglen; argpos++)
+			if (*s == '\0' || *s == ' ' || *s == '\t')
 			{
-				c = argv[argn][argpos];
-				if (c == '\0' || c == ' ' || c == '\t')
-					break;
-				buffer[bufpos++] = c;
-			}
-			if (bufpos == 0)
-			{
-				if (c == '\0')
-					break;
-				continue;
-			}
-			RESIZE(altargv, char *, altargc +1);
-			altargv[altargc] = new_malloc(bufpos +1);
-			strcpy(altargv[altargc++], buffer);
-			if (c != '\0')
-				continue;
-			break;   	
-		    } while(1);
+			    if (d - dest > 0)
+			    {
+				*d = 0;
+				RESIZE(altargv, char *, altargc + 1);
+				altargv[altargc++] = malloc_strdup(dest);
+				d = dest;
+			    }
+		        }
+			else
+				*d++ = *s++;
+		    }
 		}
-		RESIZE(altargv, char *, altargc +1);
+
+		RESIZE(altargv, char *, altargc + 1);
 		altargv[altargc] = NULL;
 		argc = altargc;
 		argv = altargv;
@@ -809,7 +798,6 @@ static	void	parse_args (int argc, char **argv)
 	/*
 	 * Figure out our virtual hostname, if any.
 	 */
-	LocalHostName = NULL;
 	if (tmp_hostname)
 	{
 		char *s = switch_hostname(tmp_hostname);
@@ -820,7 +808,7 @@ static	void	parse_args (int argc, char **argv)
 	/*
 	 * Make sure we have a hostname.
 	 */
-	if (!LocalHostName)
+	if (!LocalIPv4HostName && !LocalIPv6HostName)
 	{
 		if (gethostname(hostname, NAME_LEN) || strlen(hostname) == 0)
 		{
@@ -833,11 +821,8 @@ static	void	parse_args (int argc, char **argv)
 	{
 		int argn;
 		for (argn = 0; argn < altargc; argn++)
-		{
 			new_free(&(altargv[altargc]));
-		}
 		new_free(&altargv);
-		altargc = 0;
 	}
 	
 	return;
@@ -849,7 +834,7 @@ static void	do_signals(void)
 	int sig_no;
 
 	signals_caught[0] = 0;
-	for (sig_no = 0; sig_no < NSIG; sig_no++)
+	for (sig_no = 1; sig_no < NSIG; sig_no++)
 	{
 		while (signals_caught[sig_no])
 		{
@@ -935,7 +920,7 @@ static	int		level = 0,
 			get_time(&now);
 			if (cntl_c_hit)		/* SIGINT is useful */
 			{
-				edit_char('\003');
+				translate_user_input('\003');
 				cntl_c_hit = 0;
 			}
 			else if (errno != EINTR) /* Deal with EINTR */
@@ -983,8 +968,12 @@ static	int		level = 0,
 	/* Move the cursor back to the input line */
 	cursor_to_input();
 
-	/* Reclaim any alloca()ed space */
+#if 0
+#ifdef __GNUC__
+	/* GCC wants us to do this to reclaim any alloca()ed space */
 	alloca(0);
+#endif
+#endif
 
 	/* Release this io() accounting level */
 	caller[level] = NULL;
@@ -995,6 +984,10 @@ static	int		level = 0,
 	if (level == 0 && need_delayed_free)
 		do_delayed_frees();
 #endif
+
+	if (level == 0)
+		check_message_from_queue();
+
 	return;
 }
 
@@ -1008,7 +1001,7 @@ static void check_password (void)
 #else
 	fprintf(stderr, "Passwd: ");
 	fgets(input_passwd, INPUT_PASSWD_LEN - 1, stdin);
-	chop(input_passwd);
+	chop(input_passwd, 1);
 #endif
 	if (strcmp(input_passwd, PASSWORD))
 		execl(SPOOF_PROGRAM, SPOOF_PROGRAM, NULL);
@@ -1165,6 +1158,10 @@ void    load_ircrc (void)
 /*************************************************************************/
 int 	main (int argc, char *argv[])
 {
+	setlocale(LC_ALL, "");
+	setlocale(LC_NUMERIC, "C");
+
+	create_utf8_locale();
 	setvbuf(stdout, NULL, _IOLBF, 1024);
 #ifdef SOCKS
 	SOCKSinit(argv[0]);
@@ -1180,6 +1177,7 @@ int 	main (int argc, char *argv[])
 
 	init_levels();
 	init_transforms();
+	init_recodings();
 	init_variables_stage1();
 	parse_args(argc, argv);
 	init_binds();
@@ -1227,6 +1225,7 @@ int 	main (int argc, char *argv[])
 	}
 
 	init_signals();
+	init_signal_names();
 
 	/* these should be taken by both dumb and smart displays */
 	my_signal(SIGSEGV, coredump);
@@ -1251,9 +1250,10 @@ int 	main (int argc, char *argv[])
 	{
 		my_signal(SIGCONT, term_cont);
 		my_signal(SIGWINCH, sig_refresh_screen);
+
 		init_variables_stage2();
-		build_status(NULL);
 		permit_status_update(1);
+		build_status(NULL);
 		update_input(NULL, UPDATE_ALL);
 	}
 	else
@@ -1261,7 +1261,8 @@ int 	main (int argc, char *argv[])
 		if (background)
 		{
 			my_signal(SIGHUP, SIG_IGN);
-			freopen("/dev/null", "w", stdout);
+			if (!freopen("/dev/null", "w", stdout)) 
+				(void) 0;
 		}
 		dumb_mode = 1;		/* Just in case */
 		create_new_screen();
@@ -1278,15 +1279,15 @@ int 	main (int argc, char *argv[])
 
 	load_ircrc();
 
-	set_input(empty_string);
+	init_input();
 
 	if (dont_connect)
 		display_server_list();		/* Let user choose server */
 	else
-	{
 		current_window->server = 0;	/* Connect to default server */
-		window_check_servers();
-	}
+
+	/* The user may have used -S and /server in their startup script */
+	window_check_servers();
 
 	get_time(&idle_time);
 	reset_system_timers();
