@@ -1,11 +1,10 @@
-/* $EPIC: vars.c,v 1.109 2010/05/30 01:12:01 jnelson Exp $ */
 /*
  * vars.c: All the dealing of the irc variables are handled here. 
  *
  * Copyright (c) 1990 Michael Sandroff.
  * Copyright (c) 1991, 1992 Troy Rollo.
  * Copyright (c) 1992-1996 Matthew Green.
- * Copyright  1993, 2003 EPIC Software Labs.
+ * Copyright 1993, 2003 EPIC Software Labs.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -54,13 +53,13 @@
 #include "stack.h"
 #include "dcc.h"
 #include "keys.h"
-#include "translat.h"
 #include "timer.h"
 #include "clock.h"
 #include "mail.h"
 #include "reg.h"
 #include "commands.h"
 #include "ifcmd.h"
+#include "ssl.h"
 
 /*
  * The VIF_* macros stand for "(V)ariable.(i)nt_(f)lags", and have been
@@ -301,10 +300,11 @@ void 	init_variables_stage1 (void)
 #define DEFAULT_DEFAULT_REALNAME NULL
 	VAR(DEFAULT_REALNAME, 		STR,  NULL);
 #define DEFAULT_DEFAULT_USERNAME NULL
-	VAR(DEFAULT_USERNAME, 		STR, NULL);
+	VAR(DEFAULT_USERNAME, 		STR,  NULL);
 	VAR(DISPATCH_UNKNOWN_COMMANDS,	BOOL, NULL);
 	VAR(DISPLAY, 			BOOL, NULL);
 	VAR(DO_NOTIFY_IMMEDIATELY, 	BOOL, NULL);
+	VAR(FIRST_LINE,			STR,  NULL);
 	VAR(FLOATING_POINT_MATH, 	BOOL, NULL);
 	VAR(FLOATING_POINT_PRECISION,	INT,  NULL);
 	VAR(FLOOD_AFTER, 		INT,  NULL);
@@ -315,7 +315,6 @@ void 	init_variables_stage1 (void)
 	VAR(FLOOD_USERS,		INT,  NULL);
 	VAR(FLOOD_WARNING,		BOOL, NULL);
 	VAR(HIDE_PRIVATE_CHANNELS,	BOOL, update_all_status_wrapper);
-	VAR(HIGH_BIT_ESCAPE,		INT,  set_meta_8bit);
 	VAR(HOLD_SLIDER,		INT,  NULL);
 	VAR(INDENT,			BOOL, set_indent);
         VAR(INPUT_INDICATOR_LEFT,	STR, NULL);
@@ -329,7 +328,7 @@ void 	init_variables_stage1 (void)
 #define DEFAULT_LOAD_PATH NULL
 	VAR(LOAD_PATH,			STR,  NULL);
 	VAR(LOG,			BOOL, logger);
-	VAR(LOGFILE,			STR,  NULL);
+	VAR(LOGFILE,			STR,  set_logfile);
 #define DEFAULT_LOG_REWRITE NULL
 	VAR(LOG_REWRITE,		STR,  NULL);
 	VAR(MAIL,			INT,  set_mail);
@@ -372,6 +371,7 @@ void 	init_variables_stage1 (void)
 	VAR(SHOW_CHANNEL_NAMES, BOOL, NULL);
 	VAR(SHOW_NUMERICS, BOOL, NULL);
 	VAR(SHOW_STATUS_ALL, BOOL, update_all_status_wrapper);
+	VAR(SSL_ROOT_CERTS_LOCATION, STR, set_ssl_root_certs_location);
 	VAR(STATUS_AWAY, STR,  build_status);
 	VAR(STATUS_CHANNEL, STR,  build_status);
 	VAR(STATUS_CHANOP, STR,  build_status);
@@ -446,10 +446,7 @@ void 	init_variables_stage1 (void)
 	VAR(SUPPRESS_FROM_REMOTE_SERVER, BOOL, NULL);
 	VAR(SWITCH_CHANNELS_BETWEEN_WINDOWS, BOOL, NULL);
 	VAR(TERM_DOES_BRIGHT_BLINK, BOOL, NULL);
-#define DEFAULT_TRANSLATION NULL
-	VAR(TRANSLATION, STR,  set_translation);
-#define DEFAULT_TRANSLATION_PATH NULL
-	VAR(TRANSLATION_PATH, STR,  NULL);
+	VAR(TMUX_OPTIONS, STR,  NULL);
 	VAR(USER_INFORMATION, STR, NULL);
 	VAR(WORD_BREAK, STR,  NULL);
 #define DEFAULT_WSERV_PATH WSERV_PATH
@@ -544,9 +541,8 @@ void 	set_var_value (int svv_index, const char *value, int noisy)
 int 	set_variable (const char *name, IrcVariable *var, const char *orig_value, int noisy)
 {
 	char	*rest;
-	int	old;
 	int	changed = 0;
-	char	*value;
+	unsigned char	*value;
 	int	retval = 0;
 
 	if (orig_value)
@@ -560,7 +556,6 @@ int 	set_variable (const char *name, IrcVariable *var, const char *orig_value, i
 	    {
 		if (value && *value && (value = next_arg(value, &rest)))
 		{
-			old = var->data->integer;
 			if (do_boolean(value, &(var->data->integer))) {
 			    say("Value must be either ON, OFF, or TOGGLE");
 			    retval = -1;
@@ -573,21 +568,31 @@ int 	set_variable (const char *name, IrcVariable *var, const char *orig_value, i
 
 	    case CHAR_VAR:
 	    {
-		if (!value)
+		int	codepoint;
+
+		if (!value || !*value)
 		{
 			var->data->integer = ' ';
 			changed = 1;
+			break;
 		}
-		else if (value && *value && (value = next_arg(value, &rest)))
+
+		if ((codepoint = next_code_point((const unsigned char **)&value, 0)) == -1)
 		{
-			if (strlen(value) > 1) {
-			    say("Value of %s must be a single character", name);
-			    retval = -1;
-			} else {
-			    var->data->integer = *value;
-			    changed = 1;
-			}
+			say("New value of %s could not be determined", name);
+			retval = -1;
+			break;
 		}
+
+		if (codepoint_numcolumns(codepoint) != 1)
+		{
+			say("New value of %s must be exactly 1 column wide", name);
+			retval = -1;
+			break;
+		}
+
+		var->data->integer = codepoint;
+		changed = 1;
 		break;
 	    }
 
@@ -682,97 +687,10 @@ BUILT_IN_COMMAND(setcmd)
 	if (args && *args)
 		*args++ = 0;
 
-	if (var && *var)
-	{
-		if (*var == '-')
-		{
-			var++;
-			args = (char *) 0;
-		}
-
-		/* Exact match? */
-		upper(var);
-		b = new_bucket();
-		bucket_builtin_variables(b, var);
-
-		if (b->numitems == 1 ||
-		      (b->numitems > 1 && !my_stricmp(var, b->list[0].name)))
-		{
-			thevar = (IrcVariable *)b->list[0].stuff;
-			name = b->list[0].name;
-		}
-		else
-		{
-			thevar = NULL;
-			name = NULL;
-		}
-
-		if (!thevar || !(thevar->flags & VIF_PENDING))
-		{
-			if (thevar)
-				thevar->flags |= VIF_PENDING;
-
-			if (!do_hook(SET_LIST, "%s %s", 
-					var, args ? args : "<unset>"))
-			{
-				free_bucket(&b);
-				if (thevar)
-					thevar->flags &= ~VIF_PENDING;
-				return;		/* Grabed -- stop. */
-			}
-
-			if (name)
-			{
-			    if (!do_hook(SET_LIST, "%s %s",
-					name, args ? args : "<unset>"))
-			    {
-				free_bucket(&b);
-				if (thevar)
-					thevar->flags &= ~VIF_PENDING;
-				return;		/* Grabed -- stop. */
-			    }
-			}
-
-			if (thevar)
-				thevar->flags &= ~VIF_PENDING;
-		}
-
-		/* User didn't offer at it -- do the default thing. */
-		if (thevar)
-		{
-			if (args && !*args)
-				show_var_value(name, thevar, 0);
-			else
-				set_variable(name, thevar, args, 1);
-			free_bucket(&b);
-			return;
-		}
-
-		if (b->numitems == 0)
-		{
-		    if (do_hook(UNKNOWN_SET_LIST, "%s %s", 
-					var, args ? args : "<unset>"))
-		      if (do_hook(SET_LIST, "set-error No such variable \"%s\"",
-					var))
-			say("No such variable \"%s\"", var);
-		    free_bucket(&b);
-		    return;
-		}
-
-		else if (b->numitems > 1)
-		{
-		    if (do_hook(SET_LIST, "set-error %s is ambiguous", var))
-		    {
-			say("%s is ambiguous", var);
-			for (i = 0; i < b->numitems; i++)
-			    show_var_value(b->list[i].name, 
-					(IrcVariable *)b->list[i].stuff, 0);
-		    }
-		}
-
-		free_bucket(&b);
-	}
-	else
+	/*
+	 * Show all of the /SET values if no argument was given.
+	 */
+	if (!var || !*var)
         {
 		b = new_bucket();
 		bucket_builtin_variables(b, empty_string);
@@ -782,7 +700,135 @@ BUILT_IN_COMMAND(setcmd)
 				(IrcVariable *)b->list[i].stuff, 0);
 
 		free_bucket(&b);
+		return;
         }
+
+	/*
+	 * We're committed to changing a /SET value here.
+	 */
+
+	/*
+	 * If the name is prefixed with a hyphen, it unsets the value.
+	 */
+	if (*var == '-')
+	{
+		var++;
+		args = NULL;
+	}
+
+	/*
+	 * How many /SET VARiables are matched by what the user typed? 
+	 */
+	upper(var);
+	b = new_bucket();
+	bucket_builtin_variables(b, var);
+
+	/*
+	 * Decide if this is an "exact" match or an "uknonwn" match.
+	 */
+
+	/* Either exactly correct, or only one possible match */
+	if (b->numitems == 1 ||
+	      (b->numitems > 1 && !my_stricmp(var, b->list[0].name)))
+	{
+		thevar = (IrcVariable *)b->list[0].stuff;
+		name = b->list[0].name;
+	}
+	/* Either zero or two-or-more matches */
+	else
+	{
+		thevar = NULL;
+		name = NULL;
+	}
+
+	/*
+	 * STEP 1 -- We would Allow the user to overrule the /SET via /ON SET.
+	 * You can /SET a new value in the /ON SET!
+	 */
+	if (!thevar || !(thevar->flags & VIF_PENDING))
+	{
+		if (thevar)
+			thevar->flags |= VIF_PENDING;
+
+		/*
+		 * Offer exactly what the user asked for
+		 */
+		if (!do_hook(SET_LIST, "%s %s", 
+				var, args ? args : "<unset>"))
+		{
+			free_bucket(&b);
+			if (thevar)
+				thevar->flags &= ~VIF_PENDING;
+			return;		/* Grabbed -- stop. */
+		}
+
+		/*
+		 * Offer the fully complete name, but only
+		 * if that's not what the user asked for!
+		 */
+		if (name && my_stricmp(name, var))
+		{
+		    if (!do_hook(SET_LIST, "%s %s",
+				name, args ? args : "<unset>"))
+		    {
+			free_bucket(&b);
+			if (thevar)
+				thevar->flags &= ~VIF_PENDING;
+			return;		/* Grabbed -- stop. */
+		    }
+		}
+
+		if (thevar)
+			thevar->flags &= ~VIF_PENDING;
+	}
+
+	/*
+	 * STEP 2 -- The user /SET a valid variable, but the user didn't grab it
+	 * via an /ON SET above.
+	 */
+	if (thevar)
+	{
+		/* Either set it, or display it, depending on the args */
+		if (args && !*args)
+			show_var_value(name, thevar, 0);
+		else
+			set_variable(name, thevar, args, 1);
+
+		free_bucket(&b);
+		return;
+	}
+
+	/*
+	 * If there was no match at all (as opposed to two-or-more), then 
+	 * also throw /on unknown_set.
+	 */
+	if (b->numitems == 0)
+	{
+	    if (do_hook(UNKNOWN_SET_LIST, "%s %s", 
+				var, args ? args : "<unset>"))
+	      if (do_hook(SET_LIST, "set-error No such variable \"%s\"",
+				var))
+		say("No such variable \"%s\"", var);
+	    free_bucket(&b);
+	    return;
+	}
+
+	/*
+	 * If there was two-or-more matches (ambiguous), then also throw
+	 * /on set to tell the user it was ambiguous.
+	 */
+	else if (b->numitems > 1)
+	{
+	    if (do_hook(SET_LIST, "set-error %s is ambiguous", var))
+	    {
+		say("%s is ambiguous", var);
+		for (i = 0; i < b->numitems; i++)
+		    show_var_value(b->list[i].name, 
+				(IrcVariable *)b->list[i].stuff, 0);
+	    }
+	}
+
+	free_bucket(&b);
 }
 
 /*
@@ -837,8 +883,13 @@ char 	*make_string_var_bydata (int type, void *vp)
 			ret = malloc_strdup(var_settings[data->integer]);
 			break;
 		case CHAR_VAR:
-			ret = malloc_dupchar(data->integer);
+		{
+			char utf8str[16];
+
+			ucs_to_utf8(data->integer, utf8str, sizeof(utf8str));
+			ret = malloc_strdup(utf8str);
 			break;
+		}
 		default:
 			panic(1, "make_string_var_bydata: unrecognized type [%d]", type);
 	}
@@ -1035,8 +1086,10 @@ static void    set_wserv_type (void *stuff)
 		return;		/* It's ok */
 	if (!my_stricmp(s, "XTERM"))
 		return;		/* It's ok */
+	if (!my_stricmp(s, "TMUX"))
+		return;		/* It's ok */
 
-	say("SET WSERV_TYPE must be either SCREEN or XTERM");
+	say("SET WSERV_TYPE must be either SCREEN, XTERM or TMUX");
 	new_free(&v->string);
 }
 
@@ -1159,4 +1212,13 @@ void	do_stack_set (int type, char *args)
 }
 #endif
 
+#if 0
+void    help_topics_set (FILE *f)
+{                                                                               
+        int     i;
+
+	for (i = 0; i < var_bucket->numitems; i++)
+                fprintf(f, "set %s\n", var_bucket->list[i].name);
+}
+#endif
 
